#if !defined(ASM_GENERATOR_HH)
#define ASM_GENERATOR_HH

#include <iostream>

#include "AstNode.hh"

///////////////////////////////////////////////////////////////////////////
class AsmGenerator {
///////////////////////////////////////////////////////////////////////////

public :

  AsmGenerator( std::ostream& os ) : outStream_(os) { }
  ~AsmGenerator( void ) { }

  void operator()( AstNode* program ) {
    if (program->isOperand()) {
      if (program->isVariable()) {
	outStream_ << "PUSH R" << program->getValue() << std::endl;
      } else {
	outStream_ << "PUSH " << program->getValue() << std::endl;
      }
    } else { // n->is Operator
      if (program->getOpType()==AstNode::ISEQ) {
	this->operator()(program->getOperand(AstNode::RIGHT));
	outStream_ << "POP R" 
		   << program->getOperand(AstNode::LEFT)->getValue() 
		   << std::endl;
      } else {
	this->operator()(program->getOperand(AstNode::LEFT));
	this->operator()(program->getOperand(AstNode::RIGHT));
	switch (program->getOpType()) {
	case AstNode::PLUS : outStream_ << "ADD" << std::endl; break;
	case AstNode::MINUS : outStream_ << "SUB" << std::endl; break;
	case AstNode::TIMES : outStream_ << "MUL" << std::endl; break;
	case AstNode::FRAC : outStream_ << "DIV" << std::endl; break;
	default : outStream_ << "?"; break;
	}
      }
    }
    if (program->getNext() != 0) {
      this->operator()(program->getNext());
    }
  }

private :
  std::ostream& outStream_;

}; // End of AsmGenerator


#endif
