#if !defined(AST_NODE_HH) 
#define AST_NODE_HH

///////////////////////////////////////////////////////////////////////////
class AstNode {
///////////////////////////////////////////////////////////////////////////

public :
  enum OpType { NOP, ISEQ, PLUS, MINUS, TIMES, FRAC };
  enum Branch { LEFT = 0, RIGHT = 1, NUM_SONS = 2};

  AstNode( void ) : next_(0) { }
  ~AstNode( void ) { }

  // Abstract methods (to be redefined by subclasses)
  virtual bool isOperand() = 0;
  virtual bool isVariable() = 0;
  virtual bool isOperator() = 0;

  virtual void setValue( int v ) = 0;
  virtual void setOpType( OpType op ) = 0;
  
  virtual int getValue( void ) = 0;
  virtual OpType getOpType( void ) = 0;

  virtual AstNode* getOperand( Branch br ) = 0;
  virtual void setOperand( Branch br, AstNode* son ) = 0;

  // Getter and setter for the next_ pointer
  void setNext( AstNode* n ) { next_ = n; }
  AstNode* getNext( void ) { return next_; }

private : 
  AstNode* next_;


}; // End of class AstNode


///////////////////////////////////////////////////////////////////////////
class Operand : public AstNode {
///////////////////////////////////////////////////////////////////////////

public :
  Operand( void ) : value_(0) { }
  Operand( int value ) : value_(value) { }
  ~Operand( void ) { }

  virtual bool isOperand() { return true; }
  virtual bool isVariable() { return false; }
  virtual bool isOperator() { return false; }

  virtual void setValue( int v ) { value_ = v; }
  virtual void setOpType( OpType op ) { }
  
  virtual int getValue( void ) { return value_; }
  virtual OpType getOpType( void ) { return NOP; }

  virtual AstNode* getOperand( Branch br ) { return 0; }
  virtual void setOperand( Branch br, AstNode* son ) { };

private :

  int value_;

}; // End of class Operand

///////////////////////////////////////////////////////////////////////////
class Variable : public Operand {
///////////////////////////////////////////////////////////////////////////

public :
  Variable( void ) { }
  Variable( int value ) : Operand(value) { }
  ~Variable( void ) { }

  virtual bool isOperand() { return true; }
  virtual bool isVariable() { return true; }
  virtual bool isOperator() { return false; }

};

///////////////////////////////////////////////////////////////////////////
class Operator : public AstNode {
///////////////////////////////////////////////////////////////////////////

public :
  Operator( void ) : opType_(NOP) { 
    operands_[LEFT] = operands_[RIGHT] = 0;
  }
  Operator( OpType ot ) : opType_(ot) { 
    operands_[LEFT] = operands_[RIGHT] = 0;
  }
  ~Operator( void ) { }

  virtual bool isOperand() { return false; }
  virtual bool isVariable() { return false; }
  virtual bool isOperator() { return true; }

  virtual void setValue( int v ) { }
  virtual void setOpType( OpType op ) { opType_ = op; }
  
  virtual int getValue( void ) { return 0; }
  virtual OpType getOpType( void ) { return opType_; }

  virtual AstNode* getOperand( Branch br ) { return operands_[br]; }
  virtual void setOperand( Branch br, AstNode* son ) {
    operands_[br] = son;
  }

private :
  OpType opType_;
  AstNode* operands_[NUM_SONS];

}; // End of class Operator

#endif
